from __future__ import with_statement
from _Framework.ControlSurface import ControlSurface
from _Framework.InputControlElement import *
from _Framework.ButtonElement import ButtonElement

from LiveZone import LiveZone
from LiveZoneScene import LiveZoneScene
from LiveZoneClipSlot import LiveZoneClipSlot
from FingerboardButton import FingerboardButton
from fingerboardcodes import *
from livezonelayout import *
from colorlookup import *
import sysex
from ParameterSlot import ParameterSlot

import diagnostic

class Ztar(ControlSurface):
	def __init__(self, c_instance):
		ControlSurface.__init__(self, c_instance)
                diagnostic.logMessage = self.log_message
                self.rotateButton = None
                self.rotateSlot = None
                with self.component_guard():
                        self._session = None
                        self._sessionLayout = LiveZoneLayout(NUM_STRINGS, NUM_FRETS)
                        self._sentInitialColorSysex = False
                        self.setupLiveZone()
                        self.set_highlighting_session_component(self._session)
	        
	def disconnect(self):
		self._session = None
		ControlSurface.disconnect(self)
		return None
	
	def refresh_state(self):
		self.log_message('Refreshing...')
                with self.component_guard():
                        for control in self.controls:
                                control.clear_send_cache()

                self._send_midi((144 + FUNCTION_CHANNEL, FUNCTIONCODES['clear-frame'], 0))
                # If the whole session needs to be refreshed, we can't assume the color lookup reflects the current
                # state of Live
                self.initializeColors()

                for component in self.components:
                        component.update()
	
	def setupLiveZone(self):
		# Initialize the color lookup table - no callback at first, to give the clip slots a chance to register
		# their colors without the callback sending out a separate sysex for all of them
		sharedColorInfo['lookup'] = ColorLookup()
		
		# Create the session
		self.log_message("Rev 34.1")
		self.log_message("%d tracks, %d scenes" % (self._sessionLayout.nTracks, self._sessionLayout.nScenes))
		self._session = LiveZone(self._sessionLayout.nTracks, self._sessionLayout.nScenes)

		# Button parameters
		isMomentary = True
		
		# Set up red box scroll buttons
		self._session.set_scene_bank_buttons(
			ButtonElement(isMomentary, MIDI_NOTE_TYPE, FUNCTION_CHANNEL, FUNCTIONCODES['scroll-down']),
			ButtonElement(isMomentary, MIDI_NOTE_TYPE, FUNCTION_CHANNEL, FUNCTIONCODES['scroll-up'])
		)
		self._session.set_track_bank_buttons(
			ButtonElement(isMomentary, MIDI_NOTE_TYPE, FUNCTION_CHANNEL, FUNCTIONCODES['scroll-right']),
			ButtonElement(isMomentary, MIDI_NOTE_TYPE, FUNCTION_CHANNEL, FUNCTIONCODES['scroll-left'])
		)
		
		# Set up scene launch buttons
		sceneLaunchButtons = []
		for led in self._sessionLayout.sceneLaunchLeds():
			sceneLaunchButtons.append(FingerboardButton(isMomentary, MIDI_NOTE_TYPE, led))
		
		# Set up stop track clips buttons
		stopTrackButtons = []
		for led in self._sessionLayout.stopClipsLeds():
			stopTrackButtons.append(FingerboardButton(isMomentary, MIDI_NOTE_TYPE, led))
		self._session.set_stop_clip_value(sharedColorInfo['lookup'][COLOR_RED])
		self._session.set_stop_track_clip_buttons(tuple(stopTrackButtons))
		
		# Set up stop all clips button in the lower corner of the LiveZone
		stopAllButton = FingerboardButton(isMomentary, MIDI_NOTE_TYPE, self._sessionLayout.stopAllClipsLed())
		self._session.set_stop_all_clips_button(stopAllButton)
		
		# Set up clip launch buttons
		for iScene in range(self._sessionLayout.nScenes):
			scene = self._session.scene(iScene)
			scene.set_launch_button(sceneLaunchButtons[iScene])
			scene.set_default_value(sharedColorInfo['lookup'][COLOR_GREEN])
			scene.set_triggered_value(sharedColorInfo['lookup'][COLOR_GREEN])
			
			clipLeds = self._sessionLayout.clipLaunchLeds(iScene)
			for iClipslot in range(self._sessionLayout.nTracks):
				clipslot = scene.clip_slot(iClipslot)
				
				led = clipLeds[iClipslot]
				clipslot.set_launch_button(FingerboardButton(isMomentary, MIDI_NOTE_TYPE, led))

                # Custom button elements - required to get the message to show up in receive_midi at all
                self.rotateButton = ButtonElement(isMomentary, MIDI_NOTE_TYPE, FUNCTION_CHANNEL, FUNCTIONCODES['rotate'])
                if not self.rotateSlot:
                    self.rotateSlot = ParameterSlot()
                self.rotateSlot.setButton(self.rotateButton)
    
        def rotateLiveZone(self):
                self._sessionLayout.rotateLiveZone()
                if self._session:
                        with self.component_guard():
                                self._session.disconnect()
                                self._session = None
                with self.component_guard():
                        for control in self.controls:
                                control.disconnect()
                        self.setupLiveZone()
                        self._session.set_highlighting_callback(self._set_session_highlight)

        def initializeColors(self):
		self._sentInitialColorSysex = False
		self.sendColorSysex([])		# No need to specify colors here, since the first call will send the whole table
		sharedColorInfo['lookup'].addColorCallback(self.sendColorSysex)
	
	def sendColorSysex(self, addedColors):
		"""Responds when a color is added to the shared lookup table by one of the clip slots."""
		if not self._sentInitialColorSysex:
			# If we haven't sent a color sysex yet, then whatever's in the lookup table needs to be sent. Pack it
			# all into one message so as not to annoy the Ztar with a hundred tiny messages.
			
			# Allocate 3 bytes for each mapped color
			rgbColors, lookups = zip(*[[rgbColor, lookup] for rgbColor, lookup in sharedColorInfo['lookup']])
			colorPackets = [0] * (max(lookups) * 3)
			for i, color in enumerate(rgbColors):
				red, green, blue = separateRgb(color)
				red, green, blue = downscaleColorValue(red, green, blue)	# Downscale from 8 bits to 5
				offset = lookups[i]*3
				colorPackets[offset:offset+3] = [red, green, blue]
			
			message = sysex.sysexFromPackets(
				[0x00, 0x01, 0x42],		# Starr Labs manufacturer ID
				[0x40, 0x00, 0x00],		# Packet ID - 0x40 for color message, 0x0000 for LED-FB type 0
				colorPackets
			)
			
			self._send_midi(tuple(message))
			self._sentInitialColorSysex = True
		else:
			# Allocate 3 bytes for each color in the update
			colorPackets = []
			for color in addedColors:
				red, green, blue = separateRgb(color)
				red, green, blue = downscaleColorValue(red, green, blue)	# Downscale from 8 bits to 5
				lookup = sharedColorInfo['lookup'][color]
				colorPackets.extend([lookup, red, green, blue])
			
			message = sysex.sysexFromPackets(
				[0x00, 0x01, 0x42],		# Starr Labs manufacturer ID
				[0x40, 0x00, 0x01],		# Packet ID - 0x40 for color message, 0x0000 for LED-FB type 1
				colorPackets
			)
			
			self._send_midi(tuple(message))
	
	def handle_sysex(self, midi_bytes):
		# The only message handled here, at this point, is the message to resize/reorient the Live Zone.
		self.log_message("Handling sysex..." + str(midi_bytes))
		self.log_message(str(midi_bytes[1:4]))
		newLiveZone = sysex.handleLiveZoneSysex(midi_bytes)
		if newLiveZone and newLiveZone['frets'] > 0 and newLiveZone['strings'] > 0:
			self.log_message("Is live zone")
			if newLiveZone['isRightHanded']:
				self._sessionLayout.orientLiveZone(HIGH_FRET, LOW_STRING)
			else:
				self._sessionLayout.orientLiveZone(LOW_FRET, HIGH_STRING)
			
			if self._sessionLayout.scenesAreFrets():
				self._sessionLayout.resizeLiveZone(newLiveZone['frets'], newLiveZone['strings'])
			else:
				self._sessionLayout.resizeLiveZone(newLiveZone['strings'], newLiveZone['frets'])
			
			# With our _sessionLayout state updated, reset the session itself with those parameters
                if self._session:
                        with self.component_guard():
                                self._session.disconnect()
                                self._session = None
                with self.component_guard():
                        for control in self.controls:
                                control.disconnect()
                        self.request_rebuild_midi_map()			
                        self.setupLiveZone()
                        self._session.set_highlighting_callback(self._set_session_highlight)

        def handle_nonsysex(self, midi_bytes):
                if tuple(midi_bytes[0:2]) == (144 + FUNCTION_CHANNEL, FUNCTIONCODES['rotate']):
                        self.rotateLiveZone()
                else:
                        super(Ztar, self).handle_nonsysex(midi_bytes)
