COLOR_LEDOFF = 0x000000
COLOR_WHITE = 0xffffff
COLOR_GREEN = 0x00ff00
COLOR_RED = 0Xff0000

# Create a site for a color lookup object that can be shared across all the clipslots and the Ztar ControlSurface
sharedColorInfo = dict({
	'lookup': None
})

class ColorLookup(object):
	def __init__(self, onColorAdd=None):
		self._colors = dict()
		self._clipCounts = dict()
		if onColorAdd:
			self._colorMapCallbacks = [onColorAdd]
		else:
			self._colorMapCallbacks = []
		
		self._maxEntries = 128
		self._availableEntries = set(range(self._maxEntries))
		
		self._registerDefaultColors()
	
	def __iter__(self):
		"""Iterating over the color lookup generates (RGB, lookup index) pairs."""
		for rgbValue in self._colors:
			yield (rgbValue, self._colors[rgbValue])
	
	def __len__(self):
		return len(self._colors)
	
	def __getitem__(self, rgbValue):
		"""Using subscripting (square brackets) on the color lookup returns the lookup index for the RGB value in 
the brackets, raising KeyError if it is not in the table."""
		return self._colors[rgbValue]
	
	def _registerDefaultColors(self):
		"""This method sets certain default colors in the lookup. These should not be overwritten regardless of the
colors found in Ableton."""
		self._colors[COLOR_LEDOFF] = 0
		self._colors[COLOR_WHITE] = 1
		self._colors[COLOR_GREEN] = 2
		self._colors[COLOR_RED] = 3
		# Setting the counters for these colors negative ensures that the releaseColor() method won't remove them
		self._clipCounts[COLOR_LEDOFF] = -1
		self._clipCounts[COLOR_WHITE] = -1
		self._clipCounts[COLOR_GREEN] = -1
		self._clipCounts[COLOR_RED] = -1
		self._onAddColor([COLOR_LEDOFF, COLOR_WHITE, COLOR_GREEN, COLOR_RED])
	
	def useColor(self, rgbColor):
		"""Registers and returns a lookup table entry for the given RGB color value. Users of the color table should
call this function when using an RGB color for the first time."""
		if rgbColor not in self._colors:
			try:
				self._colors[rgbColor] = self._newLookupEntry()
				self._clipCounts[rgbColor] = 1
				self._onAddColor([rgbColor])
			except ValueError:
				raise ValueError("Couldn't add color to lookup; table is full.")
		else:
			self._clipCounts[rgbColor] += 1
		return self._colors[rgbColor]
	
	def useColors(self, rgbColorList):
		"""Same as useColor(), but defers any callbacks until all the colors in the supplied list have been added,
then calls back with the full list. Returns a list of lookup entries for the added colors."""
		newColors = []
		lookupEntries = []
		for color in rgbColorList:
			if color not in self._colors:
				self._colors[color] = self._newLookupEntry()
				self._clipCounts[color] = 1
				newColors.append(color)
			else:
				self._clipCounts[color] += 1
			lookupEntries.append(self._colors[color])
		
		self._onAddColor(newColors)
		return lookupEntries
	
	def releaseColor(self, rgbColor):
		if rgbColor in self._clipCounts and self._clipCounts[rgbColor] > 0:
			self._clipCounts[rgbColor] -= 1
			if self._clipCounts[rgbColor] == 0:
				self._availableEntries = self._availableEntries | set(self._colors[rgbColor])
				del self._colors[rgbColor]
	
	def _onAddColor(self, addedColors):
		"""Calls any registered callbacks with a list of added RGB color values."""
		self._availableEntries = self._availableEntries - set([self._colors[color] for color in addedColors])
		for callback in self._colorMapCallbacks:
			callback(addedColors)
	
	def addColorCallback(self, callback):
		"""Adds a callback function for the _onAddColor event."""
		if callable(callback) and callback not in self._colorMapCallbacks:
			self._colorMapCallbacks.append(callback)
	
	def removeColorCallback(self, callback):
		"""Removes a callback function for the _onAddColor event."""
		if callback in self._colorMapCallbacks:
			self._colorMapCallbacks.remove(callback)
		
	def _newLookupEntry(self):
		try:
			return min(self._availableEntries)
		except ValueError:
			raise ValueError("Lookup table is full.")
