def separateRgb(rgb):
	"""Breaks an integer representing an RGB triad into three integers representing each channel individually."""
	blue = rgb % 256
	green = (rgb % 65536) / 256
	red = rgb / 65536
	return [red, green, blue]

def downscaleColorValue(red, green, blue):
	"""Used to downscale the red, green, and blue color values from Ableton's 8 bits down to the 5 supported by
the fingerboard."""
	# Coerce potentially out-of-bounds values into RGB triad range.
	for color in [red, green, blue]:
		if color < 0: color = 0
		if color > 0xff: color = 0xff
	
	# Convert color values into the closest 5-bit value
	dsBlue = int(round(blue / 8.0))
	dsGreen = int(round(green / 8.0))
	dsRed = int(round(red / 8.0))
	
	# Make sure the rounding-up didn't push the value out of allowable range
	if dsRed == 32: dsRed = 31
	if dsGreen == 32: dsGreen = 31
	if dsBlue == 32: dsBlue = 31
	
	# Don't return all zeroes for a color that wasn't actually black
	if dsRed + dsGreen + dsBlue == 0 and red + green + blue != 0:
		if red > blue and red > green:
			dsRed = 1
		elif green > red and green > blue:
			dsGreen = 1
		else:
			dsBlue = 1
	return [dsRed, dsGreen, dsBlue]

FUNCTION_CHANNEL = 15	# Actually 16, but channels are numbered from 0 in code

# Function codes use channel 16 and count downwards from 127 (values 0-23 of channel 16 are used to identify LEDs).
# These functions are sent by the fingerboard to the script.
FUNCTIONCODES = dict({
	'clear-frame':	127,
	'scroll-up': 	110,
	'scroll-down':	109,
	'scroll-right':	108,
	'scroll-left':	107,
        'rotate':       84,
})

# We turn now to functions sent from the script to the fingerboard to command LED states. In general, these are
# channel 16 MIDI note messages, where the function code counts down from 127 and the velocity field identifies
# the LED whose state is being set.

# These FUNC_ constants, besides identifying these functions within this script, also correspond to the low LED 
# address codes for these functions, and are used by the functionCodeForLed() function as a basis for computing
# the actual function code that should be sent to the fingerboard.
FUNC_BLINKON = 117
FUNC_BLINKOFF = 115
FUNC_BRIGHT = 113
FUNC_DIM = 111

def functionCodeForLed(function, led):
	"""Returns the appropriate function code for an LED. Some functions sent to the fingerboard use multiple
codes since there are more than 128 LED addresses, and the address field is only wide enough for 128."""
	offsetForLedAddr = int(led / 128)
	if function == FUNC_BLINKON:
		return FUNC_BLINKON + offsetForLedAddr
	elif function == FUNC_BLINKOFF:
		return FUNC_BLINKOFF + offsetForLedAddr
	elif function == FUNC_BRIGHT:
		return FUNC_BRIGHT + offsetForLedAddr
	elif function == FUNC_DIM:
		return FUNC_DIM + offsetForLedAddr
	else:
		# If it's not one of the functions that requires an adjustment based on the LED address, the function
		# code doesn't need to be modified.
		return function
