NUM_STRINGS = 6
NUM_FRETS = 20

BASE_CHANNEL = 15	# Actually MIDI channel 16; code indexes channels starting at 0
STRINGS_PER_ZONE = 6
LEDS_PER_CHANNEL = STRINGS_PER_ZONE * NUM_FRETS

def channelForLed(led):
	return BASE_CHANNEL

def noteForLed(led):
	return led % LEDS_PER_CHANNEL

def channelAndNoteForLed(led):
	return (channelForLed(led), noteForLed(led))

# Constants used to designate the sides of the LiveZone. "Low" and "high" are used with reference to the LED numbers;
# LEDs on the "low" fret have lower numbers than those on the "high" fret. 
LOW_STRING = 0
HIGH_STRING = 1
LOW_FRET = 2
HIGH_FRET = 3

def nextSideInRotation(side):
        if side == LOW_STRING:
                return HIGH_FRET
        elif side == HIGH_STRING:
                return LOW_FRET
        elif side == LOW_FRET:
                return LOW_STRING
        elif side == HIGH_FRET:
                return HIGH_STRING
        else:
                return None

class LiveZoneLayout(object):
	"""Class for representing the layout of a LiveZone, a collection of LEDs used to launch clips, scenes, and tracks
in Ableton Live."""
	def __init__(self, nStrings=NUM_STRINGS, nFrets=NUM_FRETS):
		self.ledCount = nStrings * nFrets
		self.nStrings = nStrings
		self.nFrets = nFrets
		
		# Default right-handed arrangement
		self.resizeLiveZone(self.nStrings - 1, self.nFrets - 1)
		self.orientLiveZone(sceneLaunch=HIGH_FRET, stopClips=LOW_STRING)
	
	def orientLiveZone(self, sceneLaunch, stopClips):
		"""Marks locations of specific LEDs in the LiveZone, so this instance knows which LEDs correspond to which
function, scene, or track. We're assuming that the clip in the first row and first column of the Ableton display will
be represented by the LED in the corner opposite the intersection of the scene-launch and stop-clips arrays."""
		self._sceneLaunchPlacement = sceneLaunch
		self._stopClipsPlacement = stopClips
		if self.scenesAreFrets():
			self.resizeLiveZone(
				min(self.nScenes, self.nFrets),
				min(self.nTracks, self.nStrings)
			)
		else:
			self.resizeLiveZone(
				min(self.nScenes, self.nStrings),
				min(self.nTracks, self.nFrets)
			)
	
        def rotateLiveZone(self):
                newSceneLaunch = nextSideInRotation(self._sceneLaunchPlacement)
                newStopClips = nextSideInRotation(self._stopClipsPlacement)
                self.resizeLiveZone(self.nTracks, self.nScenes)
                self.orientLiveZone(newSceneLaunch, newStopClips)

	def resizeLiveZone(self, nScenes, nTracks):
		self.nScenes = nScenes
		self.nTracks = nTracks
	
	def ledsOfString(self, string):
		offset = string * self.nFrets
		return [led for led in range(offset, offset + self.nFrets)]
	
	def ledsOfFret(self, fret):
		return [led for led in range(fret, fret + (self.nStrings * self.nFrets), self.nFrets)]
	
	def ledsAlongSide(self, stringOrFret):
		"""Returns a list of LED numbers along the specified string or fret."""
		if stringOrFret == LOW_STRING:
			return self.ledsOfString(0)
		elif stringOrFret == HIGH_STRING:
			if self.scenesAreFrets():
				return self.ledsOfString(self.nTracks)
			else:
				return self.ledsOfString(self.nScenes)
		elif stringOrFret == LOW_FRET:
			return self.ledsOfFret(0)
		elif stringOrFret == HIGH_FRET:
			if self.scenesAreFrets():
				return self.ledsOfFret(self.nScenes)
			else:
				return self.ledsOfFret(self.nTracks)
	
	def scenesAreFrets(self):
		return self._sceneLaunchPlacement == LOW_STRING or self._sceneLaunchPlacement == HIGH_STRING
	
	def tracksAreFrets(self):
		return self._stopClipsPlacement == LOW_STRING or self._stopClipsPlacement == HIGH_STRING
	
	def scenesAreReversed(self):
		return self._stopClipsPlacement == LOW_STRING or self._stopClipsPlacement == LOW_FRET
	
	def tracksAreReversed(self):
		return self._sceneLaunchPlacement == LOW_STRING or self._sceneLaunchPlacement == LOW_FRET
	
	def stopAllClipsLed(self):
		sceneLaunchSet = set(self.ledsAlongSide(self._sceneLaunchPlacement))
		stopClipsSet = set(self.ledsAlongSide(self._stopClipsPlacement))
		led = sceneLaunchSet.intersection(stopClipsSet).pop()
		return led
	
	def sceneLaunchLeds(self):
		leds = self.ledsAlongSide(self._sceneLaunchPlacement)[:self.nScenes + 1]
		if self._stopClipsPlacement == LOW_STRING or self._stopClipsPlacement == LOW_FRET:
			leds = leds[1:]
		elif self._stopClipsPlacement == HIGH_STRING or self._stopClipsPlacement == HIGH_FRET:
			leds = leds[:-1]
		
		if self.scenesAreReversed():
			leds.reverse()
		return leds
	
	def stopClipsLeds(self):
		leds = self.ledsAlongSide(self._stopClipsPlacement)[:self.nTracks + 1]
		if self._sceneLaunchPlacement == LOW_STRING or self._sceneLaunchPlacement == LOW_FRET:
			leds = leds[1:]
		elif self._sceneLaunchPlacement == HIGH_STRING or self._sceneLaunchPlacement == HIGH_FRET:
			leds = leds[:-1]
		
		if self.tracksAreReversed():
			leds.reverse()
		return leds
	
	def clipLaunchLeds(self, iScene):
		if self.scenesAreFrets():
			if self.scenesAreReversed():
				leds = self.ledsOfFret(self.nScenes - iScene)[:self.nTracks + 1]
			else:
				leds = self.ledsOfFret(iScene)[:self.nTracks + 1]
		else:
			if self.scenesAreReversed():
				leds = self.ledsOfString(self.nScenes - iScene)[:self.nTracks + 1]
			else:
				leds = self.ledsOfString(iScene)[:self.nTracks + 1]
		
		if self.tracksAreReversed():
			leds.reverse()
		return leds[:-1]
